; UTILITY TO CREATE A DISK FROM AN IMAGE FILE ON A USB STICK
; VER 1.0 JAN 19, 2018, J MCMULLEN, WRITTEN USING SOME OF R. MAIN'S CODE
;
; PROGRAM SYNTAX CREATR <FILENM.IMG>
; DESTINATION DRIVE MUST BE FORMATTED BEFORE CREATING
; IMAGE CREATION WILL BE IN DRIVE F3. 
; CHANGE DISK BY CHANGING "DSKWRT" AS FOLLOWS
;	DISK 0 = 04H
;	DISK 1 = 14H
;	DISK 2 = 24H
;	DISK 3 = 34H 
; IN EQUATE LINE "DSKWRT	EQU	36H		; DISK F3 WRITE COMMAND	"
; AND RECOMPILE
; PROGRAM REQUIRES:
; MDS-225: RICH'S MDS-USB board with its cable at USART socket A91 or 
; equivalent and the Hobbytronics USB Host Adapter board w/ Rev 1.14 Firmware
; For other MDS systems remap defined ports herein.
; 62K MEMORY IS IDEAL
;
; THIS UTILITY WILL CREATE A DISK FROM A USB IMAGE FILE OF
; 5125120 BYTES OR 4004 BLOCKS AVAIL ON 8" DD FLOPPY DISK
; WITH ISIS 4.1  OR 4.3 AS THE OS.
;
; IMAGE FILES ARE READ IN SEGMENTS OF 33280 BYTES (5 TRACKS) UNTIL COMPLETE
; TRACK 75 AND 76 ARE THE LAST TRACKS CREATED

; THERE ARE TIME CRITICAL ROUTINES MAINLY TO KEEP UP WITH THE INBOUND SERIAL
; DATA WHICH IS NOT THROTTLED RATHER FULL SPEED SEND REQUESTED SEGMENTS.
; ALSO SPECIFIC TIMEOUTS  ARE IMPLEMENMTED THAT WAIT FOR THE ARRIVAL OF DATA 
; REQUESTED. 
; THE HOBBYTRONICS USB HOST ADAPTER TAKES LONGER RETURNING SEGMENTS THE 
; FURTHER INTO A LARGE FILE IS GOES.
;;
;
; GENERAL
;
CR	EQU	0DH
LF	EQU	0AH
;
; ISIS CALLS
;
OPEN	EQU	0
CLOSE	EQU	1
READ	EQU	3
WRITE	EQU	4
EXIT	EQU	9
ERROR	EQU	12
;
OPCPL	EQU	4
DSTS	EQU	78H
DTYPE	EQU	79H
RSTS	EQU	7BH
LOWW	EQU	79H
HI	EQU	7AH
DSKRST	EQU	7FH
;
DSKWRT	EQU	36H		; DISK F3 WRITE COMMAND	
;
CRTI	EQU	0F6H		; MDS SERIAL CHANNEL 2
CRTO	EQU	0F6H
CRTS	EQU	0F7H
CRTC	EQU	0F7H
;
TTYI	EQU	0F4H		; MDS SERIAL CHAN 1, OBSOLETE TTY INTERFACE
TTYO	EQU	0F4H		; NOW THE HOST FOR THE USB INTERFACE
TTYS	EQU	0F5H
TTYC	EQU	0F5H
;
TRDY	EQU	00000001B
RRDY	EQU	00000010B
;
DTR	EQU	00000010B
;
	EXTRN	ISIS		; EXTERNAL REFERENCES TO ISIS-II SYSTEM
				; AND MDS CONSOLE
	EXTRN	CO
	EXTRN	CI
;
	CSEG
;
;
; SET THE TIMER CH0 AND TTY USART TO OPERATE AT 9600 BAUD
;
CREATR:
	LXI	SP, STACKA	; INIT SP FOR THIS PROGRAM
	MVI	A, 40H		; RESET THE USART
	OUT	TTYC
	MVI	A, 036H		; SELECT COUNTER 0 , R/L LSB-MSB, SQ WV GEN
	OUT	0F3H		; MODE 3 AND LSB FOLLOWED BY MSB
	LXI	H, 08H		; FOR 9600 BAUD
	MOV	A, L
	OUT	0F0H		; SEND TO USART TIMER FOR TTY USART CH0
	MOV	A, H
	OUT	0F0H
	MVI	A, 0CEH		; MULTIPLIER FOR 9600 BAUD
	OUT	TTYC
	MVI	A, 25H		; ENABLE TX RX AND RTS
	OUT	TTYC
;
	MVI	A,35H		; PUT DTR AND SD0 USB HIGH
	OUT	TTYC
;
; TTY INTERFACE TO USB FLASHDRIVE READY TO GO AT 9600 BAUD
;
MSG:
	LXI	H,HELLO
	CALL	PRTMSG		; DISPLAY OUT NAME
;
GETFN:
	MVI	C,READ		; CALL ISIS TO READ THE COMMAND
	LXI	D,RBLK		; LINE TO BUFFER
	CALL	ISIS
	LDA	STATUS
	ORA	A
	JNZ	ERR
;
	LXI	H,BUFFER	; WE NEED A PRIVATE COPY OF <FILENAME>
	LXI	D,FILENM
	MVI	C,128
MOVEIT:
	MOV	A,M		; MOV CHAR FROM BUFFER
	CPI	':'
	CZ	BUMPH4		; CALL TO SCOOT H OVER BY 4 TO AVOID DEVICE
	INX	H		; INR PTR
	STAX	D		; PUT IN FILENM
	INX	D		; BUMP THAT PTR
MVIT0:
	MOV	A,M		; INSPECT WHAT WE'RE POINTING AT NOW
	CPI	20H		; A SPACE?
	JZ	ZERO		; IF SO, WE GOT THE WHOLE FILENAME
	CPI	CR		; A CR?
	JZ	ZERO		; IF SO, WE GOT IT
	DCR	C		; LIMIT TO 128 BYTES MOVED
	JZ	ZERO		; JUMP TO ZERO IF LIMIT
	JMP	MOVEIT		; OTHERWISE, MORE TO DO
;
ZERO:	MVI	A,0		; WE NEED A ZERO RIGHT AFTER <FILENAME>
	STAX	D		; D POINTS TO WHERE TO PUT THE 0
;
	LXI	H, PROCED	; ASK IF ITS OK TO PROCEED
	CALL	PRTMSG
	CALL	CI		; GET YES OR NO
	CPI	'Y'
	JNZ	DONE		; EXIT IF NO
	LXI	H, NEWLIN	; RETURN CURSOR
	CALL	PRTMSG



		; READY NOW TO READ THE DATA FROM USB
;
	OUT	DSKRST		; RESETS THE DD FDC
;
LAUNCH:
	CALL	TELL		; INFORM WHAT TRACKS WE'RE DOING
	LXI	H, RDCMD	; POINT TO USB COMMAND
LNCH0:
	MOV	C,M		; SEND CMND TO USB
	CALL	TTYOUT
	INX	H		; IS NEXT A 0?
	MOV	A,M
	CPI	0
	JNZ	LNCH0		; IF ZERO, NOW SEND <FILENAME>
;
	LXI	H,FILENM	; NOW THE FILENAME
LNCH1:
	MOV	C,M		; SEND <FILENAME>
	CALL	TTYOUT
	INX	H
	MOV	A,M
	CPI	0		; END MARKER 0?
	JNZ	LNCH1
	LDA	SEGCNT		; CASE OF SEGMENT NUMBER
	CPI	1		; FIRST
	LXI	H,SEG1
	JZ	LNCH2
	CPI	2		; SECOND
	LXI	H,SEG2
	JZ	LNCH2
	CPI	3		; THIRDS
	LXI	H,SEG3
	JZ	LNCH2
	CPI	4		; FORTH
	LXI	H,SEG4
	JZ	LNCH2
	CPI	5		; FIFTH
	LXI	H,SEG5
	JZ	LNCH2
	CPI	6		; SIXTHD
	LXI	H,SEG6
	JZ	LNCH2
	CPI	7		; SEVENTH
	LXI	H,SEG7
	JZ	LNCH2
	CPI	8		; EIGTH
	LXI	H,SEG8
	JZ	LNCH2
	CPI	9		; NINTH
	LXI	H,SEG9
	JZ	LNCH2
	CPI	10		; TENTH
	LXI	H,SEG10
	JZ	LNCH2
	CPI	11		; ELEVENTH
	LXI	H,SEG11
	JZ	LNCH2
	CPI	12		; TWELFTH
	LXI	H,SEG12
	JZ	LNCH2
	CPI	13		; THIRTEENTH
	LXI	H,SEG13
	JZ	LNCH2
	CPI	14		; FOURTEENTH
	LXI	H, SEG14
	JZ	LNCH2
	CPI	15		; FIFTEENTH
	LXI	H, SEG15
	JZ	LNCH2
	CPI	16		; SIXTEENTH SEGMENT FOR TRK 75 & 76
	LXI	H, SEG16
LNCH2:
	MOV	C, M
	CALL	TTYOUT
	INX	H
	MOV	A, M
	CPI	0
	JNZ	LNCH2
	MVI	C, CR		; GIVE USB COMMAND
	CALL	TTYOUT
;
	LXI	H, DATBUFF	; WHERE USB HAS TO BUFFER IN DATA
	MVI	C, 0FFH		; INIT HSB OF WAIT TIME OUT CNTR
	LXI	D, 0FFFFH	; INIT MSB & LSB OF TIMEOUT COUNTER
L0:				; USED TO COMPENSATE FOR VARIABLE TIME THAT USB
				; TAKES TO RETURN DATA
	IN	TTYS
	ANI	RRDY
	JNZ	LBCZ		; DATA READY? YES BRING IN DATA
	DCX	D
	MOV	A,E
	ORA	D
	JNZ	L0
	LXI	D,0FFFFH
	DCR	C
	JZ 	TIMOUT
	JMP	L0		; NOPE KEEP LOOKING

L1:

	IN	TTYS
	ANI	RRDY
	JNZ	L2
	DCX	D		; COUNT DOWN TIME OUT
	MOV	A,D
	ORA	E		; BC BOTH DOWN TO ZERO?
	JZ	TRIM		; TIME OUT ESCAPE TRAP
	JMP	L1		; LOOP WITHOUT RESETING
;
LBCZ:	LXI	B,0		; INITIALIZE BYTE COUNTER TO ZERO
L2:
	IN	TTYI
	MOV	M,A		; WRITE IT TO DATBUFF
	INX	H		; BUMP POINTER
	INX	B		; BUMP COUNT
	LXI	D,0FFFFH
	JMP	L1		; RESET THE TIME OUT
;
TRIM:	
	MOV	A,C		; TEST FOR 0 BYTE COUNT RECVD, YES WE'RE DONE
	ORA	B		; OTHERWISE WRITE THE SEGMENT
	JZ	DONE
;
;	
WRTTRK:	
	LDA	TRKSPER		; TRACKS PER SEGMENT	
	MOV	B,A		; PLACE IN TRACKS TO DO COUNTER
	LXI	H,DATBUFF
	SHLD	IOPB5		; MAKE SURE IOPB HAS RIGHT BUFF ADDR
;
; BEGIN LOOP TO READ A TRACK FROM DISK, WRITE THE TRACK TO USB
;
WRTRK1:			
	LXI	H,IOPB		; LAUNCH THE FDC TO READ THE TRACK
	MOV	A,L
	OUT	LOWW
	MOV	A,H
	OUT	HI
;
TBT1:	IN	078H		; READ STATUS FOR INTERRUPT PENDING
	ANI	4
	JZ	TBT1
	IN	DTYPE		; RESULT TYPE MUST ONLY BE READ AFTER BIT2
				; IN STATUS GOES TRUE, THEN MUST READ RSLT BYTE TOO
	ANI	2
	JZ	TYPE00
	MVI	A, DSKWRT	; GET DRIVE UNIT ID AND OPCODE
	ANI	0FH		; ISOLATE DRIVE UNIT
	MOV	C,A
	IN	DTYPE		; IT WAS TYPE 10
	ANA	C		; DRIVE MUST BE RDY
	CZ	STAR
	JMP	OK
TYPE00:				; RESULT TYPE WAS 00
	IN	07BH		; RESULT BYTE
	ORA	A		; NOT 00000000B MEAN ERROR
	CNZ	STAR
OK:
	LHLD	IOPB5		; GET CURRENT BUFFER WRITE POSITION
	LXI	D, 128 * 52	; SIZE OF 1 TRACK
	DAD	D		; ADD IT TO CURRENT BUFFER POSITION
	SHLD	IOPB5		; UPDATE BUFFER START POSITION FOR NEXT TRACK WRITE
	LDA	TRACK		; GET CURRENT TRACK
	INR	A
 	STA	TRACK
	DCR	B		; TEST FOR TRACKS SEGMENT DONE
	JNZ	WRTRK1		; DO THE NEXT TRACK
NXTSEG:
	LDA	SEGCNT		; GET CURRENT SEGMENT COUNT
	INR	A		; BUMP, SAVE AND GO DO IT
	STA	SEGCNT
	CPI	17		; 512512 BYTES
	JZ 	DONE		; ALL TRACKS WRITTENHED
	JMP	LAUNCH
;
PRTMSG:				; DISPLAYS MESSAGES TILL ZERO CHARACTER FOUND
	MOV	A,M
	ORA	A
	RZ
	MOV	C,A
	CALL	CO		; MESSAGE TO CONSOLE OUTPUT
	INX	H		; POINT TO NEXT CHARACTER
	JMP	PRTMSG
;
;
TTYOUT:				; WRITES A CHAR TO THE TTY USART AND THE USB
	IN	TTYS
	ANI	TRDY
	JZ	TTYOUT
	MOV	A,C
	OUT	TTYO
	RET
;
TTYIN:				; READS A CHAR FROM THE USB THROUGH TTY USART
	IN	TTYS
	ANI	RRDY
	JZ	TTYIN
	IN	TTYI
	RET
;
STAR:
	MVI	C, '*'
	CALL	CO		; TYPE A "*" TO CO IF ERROR, BUT CONTINUE
	RET
;
BUMPH4:	INX	H		; MOVE PAST ":Fx:" IN USER ENTERED FILENAME
	INX	H
	INX	H
	INX	H
	MOV	A,M
	RET
;
;
TIMOUT:				; SEND TIME OUT OF USB
	LXI	H,TMOUT
TMO1:	MOV	C,M
	CALL	CO
	INX	H
	MOV	A,M
	CPI	0
	JNZ	TMO1
;
DONE:
	MVI	C,CR
	CALL	CO
	MVI	C,LF
	CALL	CO
	MVI	C,EXIT
	LXI	D,XBLK
	CALL	ISIS
				; GONE TO ISIS
;
ERR:
	LXI	H, ERRMSG	; FLAG ERROR TO CONSOLE
ERR0:	MOV	A,M
	ORA	A
	JZ	ERR1
	MOV	C,A
	CALL	CO
	INX	H
	JMP	ERR0
;
ERR1:
	MVI	C, ERROR
	LXI	D, EBLK
	CALL	ISIS
	LXI	H, ERROR
	JMP	DONE
;
TELL:
	LDA	SEGCNT
	CPI	16		; CHECK FOR SECMENT 16, SHORT LAST SEGMENT
	JNZ	TELL0		; BYPASS MODIFICATION IF TRACK IS NOT 74
	MVI	A,2		; IF SEG 16, CHANGE TRACKS PER SEGMENT TO WRITE
	STA	TRKSPER		; & CHANGE THE DISPLAYED "TO/FROM" FROM 9
				; TRACKS SEPERATION TO 7
	MVI	A,1
	STA	FROMTO		; 
TELL0:	LDA	TRACK		; GET TRACK CURRENT TRACK
	ADD	A		; DOUBLE IT
	LXI	H,TRKTBL	; INDEX TO ASCII TABLE START
	MOV	E,A
	MVI	D,0
	DAD	D
	MOV	A,M		; GET MSB OF ASCII TRACK NUMBER
	STA	TRKNUM		; PUT INTO PROGRESS MESSAGE
	INX	H		; GET LSB
	MOV	A,M	
	STA	TRKNUM+1	; PUT IN MESSAGE		
	LDA	FROMTO		; GET "TO" TRACK NUMBER FROM TABLE, D=0
	MOV	E,A		; POSITION FOR DAD
	DAD	D
	MOV	A,M		; GET MSB OF "TO" ASCII TRACK NUMBER
	STA	TRKNUM+6	; PUT IN PROGRESS MESSAGE
	INX	H
	MOV	A,M		; GET MSB
	STA	TRKNUM+7	; PUT IN MESSAGE
	LXI	H,TRKMSG	; DISPLAY OUR PROGRESS
	CALL	PRTMSG
	RET
;
DSEG
;
RDCMD:	DB	'$READ ',0	; STRING USB NEEDS FOR TYPE CMD
SEG1:	DB	' 0 33280',0
SEG2:	DB	' 33280 33280',0
SEG3:	DB	' 66560 33280',0
SEG4:	DB	' 99840 33280',0
SEG5:	DB	' 133120 33280',0
SEG6:	DB	' 166400 33280',0
SEG7:	DB	' 199680 33280',0
SEG8:	DB	' 232960 33280',0
SEG9:	DB	' 266240 33280',0
SEG10:	DB	' 299520 33280',0
SEG11:	DB	' 332800 33280',0
SEG12:	DB	' 366080 33280',0
SEG13:	DB	' 399360 33280',0
SEG14:	DB	' 432640 33280',0
SEG15:	DB	' 465920 33280',0
SEG16:	DB	' 499200 13312',0
;
SEGCNT:	DB	1
TRKSPER:DB	5		; TRACKS PER SEGMENT EXCEPT FOR LAST
FROMTO:	DB	7		; TO/FROM TRACK NUMBER SEPERATION IN TRKTBL
;
TRKTBL:	DB	'00010203040506070809'
	DB	'10111213141516171819'
	DB	'20212223242526272829'
	DB	'30313233343536373839'
	DB	'40414243444546474849'
	DB	'50515253545556575859'
	DB	'60616263646566676869'
	DB	'70717273747576??????'
;
;
;
HELLO:	DB	CR,LF,'DISK CREATOR FROM USB IMAGE UTILITY, V2.0'
	DB	CR,LF,' CREATR <filenm.img>'
	DB	'....RECOMMEND FORMATTING FIRST...',CR,LF
	DB	CR,LF,'"*" IN PROGRESS FLAGS A DISK ERROR OCCURRED',CR,LF,0
PROCED:	DB	'CONTINUE ? Y/N',0
TMOUT:	DB	'TIMOUT ERROR WAITING FOR USB DATA',CR,LF,0
ERRMSG:	DB	'ISIS ERROR ENCOUNTERED, EXITING',CR,LF,0
TRKMSG:	DB	CR,'WRITING TRACKS '
TRKNUM:	DB	30H,30H,' TO '
	DB	30H,30H
NEWLIN:	DB	CR,LF,0
;
; SBC202 DISK IOPB
IOPB:	DB	0		; INTERRUPTS ON
	DB	DSKWRT		; WRITE F3
	DB	52		; 52 SECTORS AT A TIME
TRACK:	DB	0		; TRACK NUMBER
	DB	1		; START AT SECTOR 1
IOPB5:	DW	1		; DATA BUFFER
;
OBLK:
	DW	WAFT
	DW	BUFFER
	DW	2		; 2, FILE IS OPENED FOR OUTPUT(WRITE)
	DW	0
	DW	STATUS
;
CBLK:
CAFT:	DW	WAFT		; WE CLOSE WHAT WE OPENED, WAS ONLY 1
	DW	STATUS
;
RBLK:
OAFT:
RAFT:	DW	1		; OPEN PUTS THE AFT HERE FOR READ OPS
RDPTR:	DW	BUFFER		; WHERE TO PUT READ DATA
	DW	512		; READS WILL GET 4 BLOCKS, 512 BYTES
	DW	ACTUAL
	DW	STATUS
;
ACTUAL:	DS	2
;
WBLK:				
WAFT:	DS	2
	DW	DATBUFF		; WHERE TO GET THE DATA TO WRITE
WCNT:	DW	0000		; SET BY CODE TO SIZE OF DATA RECEIVED
	DW	STATUS
;
XBLK:	DW	STATUS
EBLK:
STATUS:	DS	2
	DW	STATUS
;
BUFFER:	DS	512
;
FILENM:	DS	128
;
	DS	10
STACKA	EQU	$
;
DATBUFF:DS	38400		; USB DATA IMPORT BUFFER
;
END		CREATR
;
