;	USB INTERACTIVE INTERFACE VERSION 1.0 NOV. 1, 2016 R.MAIN
;		DESIGNED FOR THE MDS-225. 
;		OTHER SYSTEMS, CHECK PORT ASSIGNMENTS
;		USES THE R. MAIN'S ADAPTATION FOR THE HOBBYTRONICS
;		USB HOST ADAPTER
;	REWRITE: VERSION 2.0 DEC 19 2017 J. MCMULLEN
;		ADDS FULL DIRECTORY SUPPORT AND MANAGEMENT
;		FOR DISPLAY OR IMPORT/EXPORT
;		ADDED SCREEN PAUSE TO IMPROVE DIRECTORY AND FILE LISTING
;		READABILTY.
;
;		VERSION 2.0 IS COMPATABLE WITH VERSION 1.13 & 1.14 
;		HOBBYTRONICS USB HOST ADAPTER FIRMWARE.
;
;	USAGE: USBJ <command>         USBJ <?> for help
;
;
CR	EQU	0DH
LF	EQU	0AH
BUFSIZ	EQU	40960
;
; ISIS CALLS
;
OPEN	EQU	0
CLOSE	EQU	1
READ	EQU	3
WRITE	EQU	4
EXIT	EQU	9
ERROR	EQU	12
;
CRTI	EQU	0F6H	; MDS SERIAL CHANNEL 2
CRTO	EQU	0F6H
CRTS	EQU	0F7H
CRTC	EQU	0F7H
;
TTYI	EQU	0F4H	; MDS SERIAL CHANNEL 1, OBSOLETE WITH TTY INTERFACE
TTYO	EQU	0F4H
TTYS	EQU	0F5H
TTYC	EQU	0F5H
;
TRDY	EQU	00000001B
RRDY	EQU	00000010B
;
DTR	EQU	00000010B
;
	EXTRN	ISIS	; EXTERNAL REFERENCES TO ISIS-II SYSTEM AND MDS CONSOLE
	EXTRN	CO
	EXTRN	CI
;
	CSEG
;
;
; SET THE TIMER CH0 AND TTY USART TO OPERATE AT 9600 BAUD
;
USB:	
	LXI	SP, STACKA	; SET PRIVATE STACK POINTER
	MVI	A,040H		; RESET THE USART
	OUT	TTYC
;
	MVI	A,036H		; SELECT COUNTER 0, R/L LSB-MSB, SQ WV GEN
	OUT	0F3H		; MODE 3 AND LSB FOLLOWED BY MSB
;
	LXI	H,08H		; FOR 9600 BAUD
	MOV	A,L
	OUT	0F0H		; SEND TO THE TIMER FOR TTY USART CH0
	MOV	A,H
	OUT	0F0H
;
	MVI	A,0CEH		; MULTIPLIER FOR 9600 BAUD
	OUT	TTYC
	MVI	A,025H		; ENABLE TX, RX, AND RTS
	OUT	TTYC
;
	MVI	A,35H		; PUT DTR AND SD0 USB HIGH
	OUT	TTYC
;
; TTY INTERFACE TO USB FLASHDRIVE READY TO GO AT 9600 BAUD
;
MSG:
	LXI	H,HELLO		; SIGN ON 
MSG1:
	MOV	A,M
	ORA	A
	JZ	GETFN
	MOV	C,A
	CALL	CO		; MESSAGE TO CONSOLE OUTPUT
	INX	H
	JMP	MSG1
;


GETFN:
	MVI	C,READ		; CALL ISIS TO READ THE USB COMMAND
	LXI	D,RBLK		; FROM THE LINE BUFFER
	CALL	ISIS
	LDA	STATUS
	ORA	A
	JNZ	ERR
;
	LXI	H,BUFFER
CMD:
	MOV	C,M		; GET CHARACTER TO SEND
	CALL	TTYOUT		; SEND IT TO USB
	MOV	A,M		; DID WE SEND CR?
	CPI	CR		; CR AT END OF COMMAND LINE?
	JZ	CMD1		; IF NO, LOOP
	CPI	'?'		; TEST IF HELP REQUESTED
	JZ	HELP		; YES, PRINT LIST OF EXAMPLE COMMANDS
	INX	H
	JMP	CMD
CMD1:
	LXI	H,BUFFER	; USB CMND SENT.... START FILLING BUFFER
	LXI	B,BUFSIZ	; MAX BUF SIZE COUNTDOWN TO PREVENT OVERFLOWS
	LXI	D,0FFFFH	; USB IDLE COUNTDOWN
	PUSH	B		; SAVE BUF SIZE COUNTDOWN
LISTEN:
	IN	TTYS
	ANI	RRDY
	JNZ	CHARREC		; GOT A CHARACTER
	DCX	D		; TIMER COUNT DOWN
	MOV	A,E		; TEST TIMER FOR 0
	ORA	D
	JNZ	LISTEN
	POP	B		; RESTORE STACK
	JMP	USBEND		; IF ZERO, WE'RE DONE
CHARREC:
	IN	TTYI
	ANI	07FH		; MASK PARITY
	MOV	M,A		; SAVE IN BUFFER
	INX	H
	LXI	D,0FFFFH	; RESET TIMER
	POP	B		; GET MAX BUFFER SIZE BACK
	DCX	B		; ONE LESS PER CHARACTER RECEIVED TILL ZERO
	MOV	A,C
	ORA	B
	JZ	EXCESS		;  MAX BUFF OVERFLOW PREVENTION
	PUSH	B		; SAVE OVERFLOW COUNTER
	JMP	LISTEN
;
EXCESS:				; BUFFER OVERFLOW PROTECTION
				; BLEEDS OFF USB DATA IN EXCESS OF BUF SIZ
				; KEEPING USB FROM HANGING WAITING TO BE READ
	MVI	A,20H		; FILL LAST USABLE USB DATA IN BUFFER
	MOV	M,A		
EXCSS0:	IN	TTYS		; CONTINUE TO READ USB DATA
	ANI	RRDY		; BUT WE DONT USE IT
	JNZ	EXCSS1		; GOT A CHARACTER
	DCX	D		; TIMER COUNT DOWN
	MOV	A,E		; TEST TIMER FOR 0
	ORA	D
	JNZ	EXCSS0
	JMP	PRINT		; IF ZERO, WE'RE DONE
EXCSS1:
	IN	TTYI		; GET THE CHARACTER BUT JUST DROP IT
	LXI	D,0FFFFH	; RESET TIMER
	JMP	EXCSS0
;
USBEND:
	XRA	A		; SET A ZERO, FLAG END OF USB OUTPUT
	MOV	M,A		; TELLS THE PRINT ROUTINE THE END OF USB DATA
PRINT:
	LXI	H, BUFFER +3	; SKIP PAST USB CMND RETURN JUNK
PRINT0:	MVI	E,24		; PAUSE AFTER 24 LINES, JUST SHOW SCRN AT A TME
PRINT1:
	MOV	C,A
	CALL	CO
	MOV	A,M		; WAS IT A ZERO?
	CPI	0		; 
	JZ	PRINT5		; YES, END OF DATA FOUND
	CPI	LF		; COUNT UP THE LF's FOR PAUSE
	JZ	PRINT3
PRINT2:	INX	H
	JMP	PRINT1
PRINT3:
	DCR	E		; COUNT DOWN LINE FEEDS 
	INX	H
	JNZ	PRINT1		; SCREEN STILL CAN TAKE MORE
	PUSH	H		; SAVE WHERE WE ARE IN THE BUFFER
	LXI	H, ANYKEY	; SEND HIT ANY KEY TO CONTINUE
PRINT4:
	MOV	C,M
	CALL	CO
	MOV	A,M
	CPI	0
	INX	H
	JNZ	PRINT4
	POP	H
	CALL	CI		; GET A CHARACTER, DONT CARE WHAT IT IS
	CALL	PRINT6		; DO A CR,LF
	JMP	PRINT0
;
PRINT5:
	CALL	PRINT6		; DO A CF,LF
	JMP	DONE
;
PRINT6:
	MVI	C,CR		; SEND A CR, LF	
	CALL	CO
	MVI	C,LF
	CALL	CO
	RET
;
ABORT:
	LXI	H,ERRMSG-1
ABORT0:
	INX	H
	MOV	A,M
	CPI	0
	JZ	DONE
	MOV	C,M
	CALL	CO
	JMP	ABORT0
;
TTYOUT:
	IN	TTYS
	ANI	TRDY
	JZ	TTYOUT
	MOV	A,C
	OUT	TTYO
	RET
;
TTYIN:
	IN	TTYS
	ANI	RRDY
	JZ	TTYIN
	IN	TTYI
	RET
;
HELP:
	LXI	H, HLPMSG
HELP1:	MOV	C,M
	CALL	CO
	MOV	A,M
	CPI	0
	INX	H
	JNZ	HELP1
DONE:
	MVI	C,CLOSE
	LXI	D,CBLK
	CALL	ISIS
DONE0:
	MVI	C,EXIT
	LXI	D,XBLK
	CALL	ISIS
ERR:
	MVI	C,ERROR
	LXI	D,EBLK
	CALL	ISIS
	JMP	DONE0
;
	DSEG
;
ERRMSG:	DB CR,'USER ABORTED',CR,LF,0
HELLO:	DB CR,LF,'USB INTERACTIVE INTERFACE VER 2.0',CR,LF
	DB 'USB <command>     USB <?> For help',CR,LF,0
ANYKEY:	DB CR,LF,'PRESS ANY KEY TO CONTINUE.......',0
HLPMSG:
DB CR,LF
DB 'EXAMPLES:',CR,LF
DB 'USB DIR                         Directory of current path',CR,LF
DB 'USB DIR /F <file name>          Directory for specific file name',CR,LF
DB 'USB CD <directory name>         Change directory to',CR,LF
DB '       <directory\directory\>',CR,LF
DB '       <..>                     Move up to parent directory',CR,LF
DB '       <\>                      Move up to root directory',CR,LF
DB 'USB MD <directory name>         Make directory',CR,LF
DB 'USB RD <directory name>         Delete directory',CR,LF
DB 'USB TYPE <filename>             Print a file to the screen',CR,LF
DB '                                (truncates at 40k',CR,LF
DB 'USB COPY <filename> <filename>  Copy file to file in current directory',CR,LF
DB 'USB DEL <filename>              Delete file in current directory',CR,LF
DB 'USB REN <filename> <filename>   Rename file in current directory',CR,LF
DB	CR,LF
DB 0
;
OBLK:
	DW	WAFT
	DW	BUFFER
	DW	2
	DW	0
	DW	STATUS
;
CBLK:
CAFT:	DS	2
	DW	STATUS
;
RBLK:
OAFT:
RAFT:	DW	1
	DW	BUFFER
	DW	128
	DW	ACTUAL
	DW	STATUS
;
ACTUAL:	DS	2
;
WBLK:
WAFT:	DS	2
	DW	BUFFER
WCNT:	DS	2
	DW	STATUS
;
XBLK:	DW	STATUS
EBLK:
STATUS:	DS	2
	DW	STATUS
;
BUFFER:	DS	BUFSIZ		; BIG BIG BUFFER
	DB	'          ',CR,LF,'******** BUFFER OVERFLOW ********',CR,LF,0
	DS	10		; OVERFLOW PAD
;
FILENM:	DS	128
;
FIRSTNM:DS	2
DIGITS:	DS	1
BYTECNT:DS	2
;
RESP:	DS	128
;

	DS	10
STACKA	EQU	$
;
END	USB

