; INTEL MDS XPORT TO USB STICK UTILITY
; VER 1.0 NOV 16, 2016, RICHARD MAIN author
; VER 2.0 DEC 12, 2017, JACK MCMULLEN, MAJOR REWRITE FOR EXTENDED FILE SIZES
; VER 2.1 DEC 18, 2017, CORRECTED CLOSE AFTN, SET STACK POINTER
;
; PROGRAM SYNTAX XPORTJ <DEVICE><FILENAME>
;
; PROGRAM REQUIRES:
; MDS-225: RICH'S MDS-USB board with its cable at USART socket A91 or 
; equivalent and the Hobbytronics USB Host Adapter board w/ Rev 1.14 Firmware
; For other MDS systems remap TTY defined ports herein.
; 62K MEMORY IS IDEAL
;
; THIS UTILITY WILL XPORT FILES IN SEGMENTS OF 32768 BYTES UNTIL COMPLETE.
; THERE IS NO PROGRAM LIMITATION ON MAX FILE SIZE
;
; THERE ARE TIME CRITICAL ROUTINES MAINLY TO KEEP UP WITH THE OUTBOUND DATA
; WHICH IS NOT THROTTLED RATHER STREAMED AT FULL SPEED.
;
;
; GENERAL
;
CR	EQU	0DH
LF	EQU	0AH
;
; ISIS CALLS
;
OPEN	EQU	0
CLOSE	EQU	1
READ	EQU	3
WRITE	EQU	4
EXIT	EQU	9
ERROR	EQU	12
;
CRTI	EQU	0F6H		; MDS SERIAL CHANNEL 2
CRTO	EQU	0F6H
CRTS	EQU	0F7H
CRTC	EQU	0F7H
;
TTYI	EQU	0F4H		; MDS SERIAL CHANNEL 1, OBSOLETE WITH TTY INTERFACE
TTYO	EQU	0F4H
TTYS	EQU	0F5H
TTYC	EQU	0F5H
;
TRDY	EQU	00000001B
RRDY	EQU	00000010B
;
DTR	EQU	00000010B
;
	EXTRN	ISIS		; EXTERNAL REFERENCES TO ISIS-II SYSTEM AND MDS CONSOLE
	EXTRN	CO
	EXTRN	CI
;
	CSEG
;
;
; SET THE TIMER CH0 AND TTY USART TO OPERATE AT 9600 BAUD
;
XPORTJ:	
	LXI	SP, STACKA	; SET STACK POINTER FOR THIS PROGRAM 
	MVI	A,040H		; RESET THE USART
	OUT	TTYC
;
	MVI	A,036H		; SELECT COUNTER 0, R/L LSB-MSB, SQ WV GEN
	OUT	0F3H		; MODE 3 AND LSB FOLLOWED BY MSB
;
	LXI	H,08H		; FOR 9600 BAUD
	MOV	A,L
	OUT	0F0H		; SEND TO THE TIMER FOR TTY USART CH0
	MOV	A,H
	OUT	0F0H
;
	MVI	A,0CEH		; MULTIPLIER FOR 9600 BAUD
	OUT	TTYC
	MVI	A,025H		; ENABLE TX, RX, AND RTS
	OUT	TTYC
;
	MVI	A,35H		; PUT DTR AND SD0 USB HIGH
	OUT	TTYC
;
; TTY INTERFACE TO USB FLASHDRIVE READY TO GO AT 9600 BAUD
;
MSG:
	LXI	H,HELLO
MSG1:
	MOV	A,M
	ORA	A
	JZ	GETFN
	MOV	C,A
	CALL	CO		; MESSAGE TO CONSOLE OUTPUT
	INX	H		; POINT TO NEXT CHARACTER
	JMP	MSG1
;
GETFN:
	MVI	C,READ		; CALL ISIS TO READ THE COMMAND
	LXI	D,RBLK		; LINE TO BUFFER
	CALL	ISIS
	LDA	STATUS
	ORA	A
	JNZ	ERR
;
	LXI	H,BUFFER	; WE NEED A PRIVATE COPY OF <FILENAME>
	LXI	D,FILENM
	MVI	C,128
MOVEIT:
	MOV	A,M		; MOV CHAR FROM BUFFER
	CPI	':'
	CZ	BUMPH4		; CALL TO SCOOT H OVER BY 4 TO AVOID DEVICE
	INX	H		; INR PTR
	STAX	D		; PUT IN FILENM
	INX	D		; BUMP THAT PTR
MVIT0:
	MOV	A,M		; INSPECT WHAT WE'RE POINTING AT NOW
	CPI	20H		; A SPACE?
	JZ	ZERO		; IF SO, WE GOT THE WHOLE FILENAME
	CPI	CR		; A CR?
	JZ	ZERO		; IF SO, WE GOT IT
	DCR	C		; LIMIT TO 128 BYTES MOVED
	JZ	ZERO		; JUMP TO ZERO IF LIMIT
	JMP	MOVEIT		; OTHERWISE, MORE TO DO
;
ZERO:	MVI	A,0		; WE NEED A ZERO RIGHT AFTER <FILENAME>
	STAX	D		; D POINTS TO WHERE TO PUT THE 0
;
;
OPENFL:				; OPEN THE FILE THE USER WANTED
				; THIS GETS THE AFT FOR THE TEXT NAME
	MVI	C,OPEN
	LXI	D,OBLK
	CALL	ISIS		; THIS PUT THE AFTN IN RAFT IN RBLK
	LDA	STATUS
	ORA	A
	JNZ	ERR
	LHLD    RAFT		; GET CURRENT AFTN
	SHLD	CAFT		; PUT IN CLOSE PARAMETER BLOCK
	JMP	LAUNCH		; READY NOW TO READ ALL BLOCKS IN THE FILE
;
;
READFL:
	MVI	C,READ
	LXI	D,RBLK
	CALL	ISIS
	LDA	STATUS
	ORA	A
	JNZ	ERR
	JMP	LAUNCH		; ISSUE COMMAND WITH <FILENAME>
;
BUMPH4:	INX	H
	INX	H
	INX	H
	INX	H
	MOV	A,M
	RET
;
LAUNCH:
	LXI	H,WRTMSG	; POINT TO USB COMMAND
LNCH0:
	MOV	C,M		; SEND CHAR TO USB
	CALL	TTYOUT
	INX	H		; IS NEXT A 0?
	MOV	A,M
	CPI	0
	JNZ	LNCH0		; IF ZERO, NOW SEND <FILENAME>
;
	LXI	H,FILENM	; NOW THE FILENAME
LNCH1:
	MOV	C,M		; SEND <FILENAME>
	CALL	TTYOUT
	INX	H
	MOV	A,M
	CPI	0		; END MARKER 0?
	JNZ	LNCH1
	MVI	C,' '		; PUT A SPACE AFTER FILENAME
	CALL	TTYOUT
	MVI	C,'B'		; THEN A "B" FOR BINARY MODE
	CALL	TTYOUT
	MVI	C,CR		; GIVE THE USB THE CR TO LAUNCH COMMAND
	CALL	TTYOUT
;
	MVI	B,4
USBRDY:	CALL	TTYIN
	DCR	B		; PULL IN 4 CHARACTERS
	JNZ	USBRDY		; 
;
RDDSK:	
	LXI	H,BUFFER	; GET STARTING ADDR OF DATA BUFFER
	SHLD	RDPTR		; PUT THAT IN THE READ BLOCK FOR ISIS IO
				; READY TO READ THE FIRST 32768 BYTE BLOCK
	MVI	C,READ		; READ 32768 BYTES OF THE FILE IF ITS THAT LARGE
	LXI	D,RBLK		; INTO BUFFER
	CALL	ISIS
	LDA	STATUS
	ORA	A
	JNZ	ERR
;				; NO ERROR, DATA IS GOOD
	LHLD	ACTUAL		; GET ACTUAL BYTES READ
	MOV	A,L
	ORA	H		; TEST FOR NO BYTES
	JZ	WRAPUP		; YES? FILE END REACHED OTHERWIZE
	XCHG			; PUT ACTUAL IN DE FOR LOOP COUNTER

SENDUSB:
	CALL	TELL		; DISPLAY SEGMENT SENT PROGRESS
	LXI	H,BUFFER	; START AT BEGINNING & STREAM DATA IN BUFFER TO USB
MORE:
	MOV	C,M
T1:	IN	TTYS
	ANI	TRDY
	JZ	T1
	MOV	A,C
	OUT	TTYO
	INX	H		; MOVE THRU BUFFER AND
	DCX	D		; DOWNCOUNT ACTUAL WHILE LOOPING
	MOV	A,E
	ORA	D		; IA ACTUAL AT ZERO?
	JNZ	MORE		; LOOP TO SEND MORE CHARS TO USB
	LDA	SEGCNT		; BUMP SEGMENT COUNT
	INR	A
	STA	SEGCNT
	JMP	RDDSK		; BUFFER HAS BEEN SENT
;
WRAPUP:	CALL	DTRLO		; TELL USB FILE TRANSFER COMPLETE AND SAVE FILE
	JMP	DONE
;
DTRLO:	IN	TTYS		; TOGGLE DTR AFTER WAITING FOR USART
	ANI	4
	JZ	DTRLO		; WAIT FOR TX EMPTY
	LXI	D,1000H
DTRLO0:				; GIVE THE USB A LITTLE TIME
	DCR	E
	JNZ	DTRLO0
	DCR	D
	JNZ	DTRLO0
;
	MVI	A,37H		; TOGGLE DTR AND USB SD0 LOW , TRANSFER COMPLETE
	OUT	TTYC
	MVI	A,35H		; RETURN THEM TO HIGH
	OUT	TTYC
	RET			; TIME TO WRAP IT UP
;

;
TELL:				; DECIMALIZES SEGMENT COUNT FOR DISPLAY
	XRA	A		; CLEAR ALL FLAGS FOR DAA
	LDA	SEGCNT		; GET CURRENT SEGMENT COUNT
	DAA			; MAKE DECIMAL
	PUSH	PSW		; SAVE A COPY
	ANI	0FH		; ISOLATE LSD
	ADI	30H		; MAKE IT ASCII
	STA	SEGNUM +1	; PLACE IT
	POP	PSW		; GET THE COPY BACK
	ANI	0F0H		; ISOLATE THE MSD
	RAR			; POSITION IN LOWER NIBBLE
	RAR
	RAR
	RAR
	ADI	30H		; MAKE IT ASCII
	STA	SEGNUM		; PLACE IT
	LXI	H, SEGMSG	; DISPLAY MESSAGE
TELL0:	MOV	C,M		; SEND CMMD TO USB
	CALL	CO
	INX	H		; IS NEXT A 0?
	MOV	A,M
	CPI	0
	JNZ	TELL0		; TELL WHAT SEGMENT WE'RE DOING
	RET


TTYOUT:				; WRITES A CHAR TO THE TTY USART AND THE USB
	IN	TTYS
	ANI	TRDY
	JZ	TTYOUT
	MOV	A,C
	OUT	TTYO
	RET
;
TTYIN:				; READS A CHAR FROM THE USB THROUGH TTY USART
	IN	TTYS
	ANI	RRDY
	JZ	TTYIN
	IN	TTYI
	RET
;
DONE:
	MVI	C,CLOSE
	LXI	D,CBLK
	CALL	ISIS
DONE0:
	MVI	C,EXIT
	LXI	D,XBLK
	CALL	ISIS
ERR:
	MVI	C,ERROR
	LXI	D,EBLK
	CALL	ISIS
	JMP	DONE0
;
	DSEG
;
HELLO:	DB	CR,LF,'XPORTJ - MSD FILE TO USB EXPORT UTILITY, V2.1'
	DB	CR,LF,' XPORTJ <:Fx:filename>',CR,LF
	DB	'FILES ARE EXPORTED IN 32768 BYTE SEGMENTS'
	DB	' - THERE IS NO MAX FILE SIZE'
	DB	CR,LF,CR,LF,0
;
WRTMSG:	DB	'WRITE ',0	; STRING USB NEEDS FOR WRITE
;
SEGCNT:	DB	1
SEGMSG:	DB	CR,LF,'WRITING SEGMENT '
SEGNUM:	DB	30H,30H
	DB	' TO USB...',CR,LF,0
;
;
OBLK:
	DW	RAFT
	DW	BUFFER
	DW	1		; 1, FILE IS OPENED FOR INPUT (READ)
	DW	0
	DW	STATUS
;
CBLK:
CAFT:	DS	2		; WE CLOSE WHAT WE OPENED, WAS ONLY 1
	DW	STATUS
;
RBLK:
OAFT:
RAFT:	DW	1		; OPEN PUTS THE AFT HERE FOR READ OPS
RDPTR:	DW	BUFFER		; WHERE TO PUT READ DATA
	DW	32768		; READ SIZE
	DW	ACTUAL
	DW	STATUS
;
ACTUAL:	DW	0000H		; HOW MANY BYTES WERE ACTUALLY READ
;
;
XBLK:	DW	STATUS
EBLK:
STATUS:	DS	2
	DW	STATUS
;
FILENM:	DS	128
;
	DS	10
STACKA	EQU	$		; STACK STARTS AT TOP OF ITS BUFFER
;
BUFFTOP:DW	BUFFER+38400
;
BUFFER:DS	38400		; XPORT TO USB DATA BUFFER
;
END		XPORTJ

